float4x4
    ProjectionMatrix; 
float4x4
    ModelViewMatrix;
float4x4
    TextureCoordinateTransformMatrix;
float4
    TextureModulationColor = { 1.0f, 1.0f, 1.0f, 1.0f };
    
// ~~

struct VS_INPUT
{
    float3 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_COLORED_TEXTURED_INPUT
{
    float3 Position : POSITION;
    float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_COLORED_TEXTURED_OUTPUT
{
    float4 Position : POSITION;
    float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_COLORED_INPUT
{
    float3 Position : POSITION;
    float4 Color : COLOR0;
};

struct VS_COLORED_OUTPUT
{
    float4 Position : POSITION;
    float4 Color : COLOR0;
};

struct VS_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_MULTI_TEXTURED_INPUT
{
    float3 Position : POSITION;
    float2 TextureCoordinate0 : TEXCOORD0;
    float2 TextureCoordinate1 : TEXCOORD1;
};

struct VS_MULTI_TEXTURED_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate0 : TEXCOORD0;
    float2 TextureCoordinate1 : TEXCOORD1;
};

Texture DiffuseTexture;

sampler2D DiffuseTextureSampler
{ 
    Texture = <DiffuseTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
};

Texture DiffuseTexture1;

sampler2D DiffuseTextureSampler1
{ 
    Texture = <DiffuseTexture1>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
};

// ~~

VS_OUTPUT vs_main(
    VS_INPUT input,
    const uniform bool it_uses_coordinates_transform
    )
{
    VS_OUTPUT
        output = (VS_OUTPUT) 0;
    float4x4
        wvp;
    
    wvp = mul( ModelViewMatrix, ProjectionMatrix );
    output.Position = mul( float4( input.Position, 1 ), wvp );
    
    if ( it_uses_coordinates_transform )
    {        
        output.TextureCoordinate = ( mul( float4( input.TextureCoordinate.xy, 0.0f, 1.0f ), TextureCoordinateTransformMatrix ) ).xy;        
    }
    else
    {
        output.TextureCoordinate = input.TextureCoordinate;
    }
    
    return output;
}

// ~~

VS_COLORED_TEXTURED_OUTPUT vs_colored_textured_main(
    VS_COLORED_TEXTURED_INPUT input,
    const uniform bool it_uses_coordinates_transform
    )
{
    VS_COLORED_TEXTURED_OUTPUT
        output = (VS_COLORED_TEXTURED_OUTPUT) 0;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );
    output.Position = mul( float4( input.Position, 1 ), wvp );

    if ( it_uses_coordinates_transform )
    {
    
        output.TextureCoordinate = ( mul( float4( input.TextureCoordinate.xy, 0.0f, 1.0f ), TextureCoordinateTransformMatrix ) ).xy;
    
    }
    else
    {
        output.TextureCoordinate = input.TextureCoordinate;
    }

    output.Color = input.Color;
    
    return output;
}

// ~~

VS_COLORED_OUTPUT vs_colored_main(
    VS_COLORED_INPUT input
    )
{
    VS_COLORED_OUTPUT
        output = (VS_COLORED_OUTPUT) 0;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );
    output.Position = mul( float4( input.Position, 1 ), wvp );
    output.Color = input.Color;
    
    return output;
}

// ~~

VS_MULTI_TEXTURED_OUTPUT vs_multi_textured_main(
    VS_MULTI_TEXTURED_INPUT input
    )
{
    VS_MULTI_TEXTURED_OUTPUT
        output = (VS_MULTI_TEXTURED_OUTPUT) 0;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );
    output.Position = mul( float4( input.Position, 1 ), wvp );

    output.TextureCoordinate0 = input.TextureCoordinate0;
    output.TextureCoordinate1 = input.TextureCoordinate1;
    
    return output;
}

// ~~

float4 ps_textured_main(
    float2 texture_coordinate : TEXCOORD0 
    ) : COLOR0
{
    return tex2D( DiffuseTextureSampler, texture_coordinate ); 
}

// ~~

float4 ps_modulated_multi_textured_mask_main(
    float2 texture_coordinate0 : TEXCOORD0,
    float2 texture_coordinate1 : TEXCOORD1
    ) : COLOR0
{
    float alpha = tex2D( DiffuseTextureSampler1, texture_coordinate1 ).a;
    float4 texture_color = tex2D( DiffuseTextureSampler, texture_coordinate0 )  * TextureModulationColor;
    
    texture_color.a = saturate( texture_color.a - alpha );
    
    return texture_color;
}

// ~~

float4 ps_multi_textured_mask_main(
    float2 texture_coordinate0 : TEXCOORD0,
    float2 texture_coordinate1 : TEXCOORD1
    ) : COLOR0
{
    float alpha = tex2D( DiffuseTextureSampler1, texture_coordinate1 ).a;
    float4 texture_color = tex2D( DiffuseTextureSampler, texture_coordinate0 );
    
    texture_color.a = saturate( texture_color.a - alpha );
    
    return texture_color;
}

// ~~

float4 ps_colored_textured_main(
    float4 color : COLOR0,
    float2 texture_coordinate : TEXCOORD0
    ) : COLOR0
{
    return tex2D( DiffuseTextureSampler, texture_coordinate ) * color; 
}

// ~~

float4 ps_colored_main(
    float4 color : COLOR0
    ) : COLOR0
{
    return color; 
}

// ~~

float4 ps_modulated_colored_textured_main(
    float2 texture_coordinate : TEXCOORD0
    ) : COLOR0
{
    return tex2D( DiffuseTextureSampler, texture_coordinate ) * TextureModulationColor; 
}

// ~~

technique TexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main( false );
        PixelShader = compile ps_3_0 ps_textured_main();
    }
}

// ~~

technique ColoredInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_colored_main();
        PixelShader = compile ps_3_0 ps_colored_main();
    }
}

// ~~

technique ColoredTexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_colored_textured_main( false );
        PixelShader = compile ps_3_0 ps_colored_textured_main();
    }
}

// ~~

technique ModulatedTexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main( false );
        PixelShader = compile ps_3_0 ps_modulated_colored_textured_main();
    }
}

// ~~

technique ModulatedTransformedTexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_colored_textured_main( true );
        PixelShader = compile ps_3_0 ps_modulated_colored_textured_main();
    }
}

// ~~

technique TransformedTexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main( true );
        PixelShader = compile ps_3_0 ps_textured_main();
    }
}

// ~~

technique ModulatedMultiTexturedMaskInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_multi_textured_main();
        PixelShader = compile ps_3_0 ps_modulated_multi_textured_mask_main(); 
    }
}

// ~~

technique MultiTexturedMaskInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_multi_textured_main();
        PixelShader = compile ps_3_0 ps_multi_textured_mask_main(); 
    }
}