// .. CONSTANTS

float3
    AmbientLightColor = { 0.0f, 0.0f, 0.0f },
    LightDirection0 = { 0.0f, -1.0f, -1.0f },
    LightDirection1 = { 1.0f, 0.0f, 0.0f },
    LightDiffuseColor0 = { 0.f, 0.8f, 0.f },
    LightDiffuseColor1 = { 1.f, 0.1f, 0.0f };
float
    SpecularIntensity0 = 0.5f,
    SpecularIntensity1 = 0.5f;
    
// .. TYPES

struct VS_TEXTURED_INPUT
{
    float2 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_TEXTURED_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

Texture NormalTexture;

sampler2D NormalTextureSampler
{ 
    Texture = <NormalTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
};

// .. FUNCTIONS

VS_TEXTURED_OUTPUT vs(
    VS_TEXTURED_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
        
    output.Position = float4( input.Position, 0, 1 );
    output.TextureCoordinate = input.TextureCoordinate;
    
    return output;
}

// ~~

float luminosity( float3 color )
{
    return dot( color, float3( 0.2126, 0.7152, 0.0722 ) );    
}

float4 ps(
    float2 texture_coordinate : TEXCOORD0
   ) : COLOR0
{
    float4
        result = { 0.0f, 0.0f, 0.0f, 0.0f };
    float3
        normal,
        light_direction,
        reflected,
        view;
    float
        n_dot_l,
        r_dot_v,
        specular_intensity;

    result.rgb += AmbientLightColor;

    normal.xy = -1.0f + 2.0f * tex2D( NormalTextureSampler, texture_coordinate ).rg;
    normal.z = sqrt( 1.0f - normal.x * normal.x - normal.y * normal.y );

    //:TODO: get real view vector
    view = float3( 0, 0, 1 );

    light_direction = normalize( -LightDirection0 );
    n_dot_l = saturate( dot( normal, light_direction ) );
    result.rgb += n_dot_l * LightDiffuseColor0;

    reflected = reflect( -light_direction, normal );
    r_dot_v = saturate( dot( reflected, view ) );
    specular_intensity = luminosity( LightDiffuseColor0 ) * SpecularIntensity0;
    //result.a += specular_intensity * pow( r_dot_v, 64 );
    
    light_direction = normalize( -LightDirection1 );
    n_dot_l = saturate( dot( normal, light_direction ) );
    result.rgb += n_dot_l * LightDiffuseColor1;

    reflected = reflect( -light_direction, normal );
    r_dot_v = saturate( dot( reflected, view ) );
    specular_intensity = luminosity( LightDiffuseColor1 ) * SpecularIntensity1;
    //result.a += specular_intensity * pow( r_dot_v, 64 );
    
    saturate( result.rgb );

    //:TODO: use color gradient lookup texture on RGB channels


    return result;
}

// ~~

technique Default
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs();
        PixelShader = compile ps_3_0 ps();
    }
}