float4x4
    ProjectionMatrix;
float4x4
    ModelViewMatrix;

// ~~

float2 LOCAL_VertexLookupTable[ 6 ]
    = { float2( -1.0f, 1.0f ), float2( 1.0f, 1.0f ), float2( -1.0f, -1.0f ), float2( -1.0f, -1.0f ), float2( 1.0f, -1.0f ), float2( 1.0f, 1.0f ) };

// ~~

struct VS_TEXTURED_INPUT
{
    float2 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
    float4 Color : COLOR0;
};

struct VS_TEXTURED_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
    float4 Color: COLOR0;
};

struct VS_VERTEX_INPUT
{
    float3 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
	float2 Size : TEXCOORD5;
	float Index : PSIZE0;
    float4 Color : COLOR0;
};

struct VS_ROTATING_VERTEX_INPUT
{
    float3 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
	float2 Size : TEXCOORD5;
	float Index : PSIZE0;
    float4 Color : COLOR0;
    float3 UpVector : TANGENT0;
    float3 LeftVector : NORMAL0;
};


Texture DiffuseTexture;

sampler2D DiffuseTextureSampler
{
  Texture = <DiffuseTexture>;
};

// .. VERTEX SHADERS

VS_TEXTURED_OUTPUT vs_main(
    VS_TEXTURED_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );

    output.Position = mul( float4( input.Position, 0, 1 ), wvp );
    output.TextureCoordinate = input.TextureCoordinate;

    output.Color = input.Color;
    return output;
}

// ~~

VS_TEXTURED_OUTPUT vs_particle_3d_point(
    VS_VERTEX_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );

    output.Position = mul( float4( input.Position, 1 ), ModelViewMatrix );

    output.Position.xy += LOCAL_VertexLookupTable[ input.Index ] * input.Size;

    output.Position = mul( output.Position, ProjectionMatrix );

    output.TextureCoordinate = input.TextureCoordinate;

    output.Color = input.Color;

    return output;
}

// ~~

VS_TEXTURED_OUTPUT vs_particle_3d_self_perpendicular(
    VS_ROTATING_VERTEX_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );

    output.Position = float4( input.Position, 1 );

    output.Position.xyz += ( LOCAL_VertexLookupTable[ input.Index ].x * input.Size.x * input.LeftVector ) 
    	+ ( LOCAL_VertexLookupTable[ input.Index ].y * input.Size.x * input.UpVector );

    output.Position = mul( output.Position, wvp );

    output.TextureCoordinate = input.TextureCoordinate;

    output.Color = input.Color;

    return output;
}

// ~~

VS_TEXTURED_OUTPUT vs_particle_3d_rotating_point(
    VS_ROTATING_VERTEX_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );

    output.Position = mul( float4( input.Position, 1 ), ModelViewMatrix );
    
    output.Position.xy += ( LOCAL_VertexLookupTable[ input.Index ].x * input.Size.x * input.LeftVector.xy ) + ( LOCAL_VertexLookupTable[ input.Index ].y * input.Size.y * input.UpVector.xy );

    output.Position = mul( output.Position, ProjectionMatrix );

    output.TextureCoordinate = input.TextureCoordinate;

    output.Color = input.Color;

    return output;
}

// .. PIXEL SHADERS

float4 ps_main(
   float2 texture_coordinate : TEXCOORD0,
   float4 color : COLOR0
   ) : COLOR0
{
    float4 result = color * tex2D( DiffuseTextureSampler, texture_coordinate ); 
    
    result.xyz *= color.a;
    
    return result;
}

// ~~

technique Default
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main();
        PixelShader = compile ps_3_0 ps_main();
    }
}

// ~~

technique Particle3DPoint
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_particle_3d_point();
        PixelShader = compile ps_3_0 ps_main();
    }
}

// ~~

technique Particle3DSelfPerpendicular
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_particle_3d_self_perpendicular();
        PixelShader = compile ps_3_0 ps_main();
    }
}

// ~~

technique Particle3DRotatingPoint
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_particle_3d_rotating_point();
        PixelShader = compile ps_3_0 ps_main();
    }
}
