// .. MACROS

#ifdef GRAPHIC_PLATFORM_Dx11
    #define TECHNIQUE technique11
    #define PS_OUTPUT_TYPE SV_Target0
    #define POSITION_INPUT_TYPE SV_POSITION
    #define VS_COMPILE_VERSION vs_4_0
    #define PS_COMPILE_VERSION ps_4_0
    #define GS_COMPILE_VERSION gs_4_0
#else
    #define TECHNIQUE technique
    #define PS_OUTPUT_TYPE COLOR0
    #define POSITION_INPUT_TYPE POSITION
    #define VS_COMPILE_VERSION vs_3_0
    #define PS_COMPILE_VERSION ps_3_0
    #define GS_COMPILE_VERSION UNSUPPORTED
#endif

// .. CONSTANTS

float4x4
    ProjectionMatrix,
    CameraMatrix; 

// .. TYPES

struct VS_COLORED_2D_INPUT
{
    float2 Position : POSITION_INPUT_TYPE;
    float4 Color : COLOR0;
};

// ~~

struct VS_COLORED_3D_INPUT
{
    float3 Position : POSITION_INPUT_TYPE;
    float4 Color : COLOR0;
};

// ~~

struct VS_COLORED_OUTPUT
{
    float4 Position : POSITION_INPUT_TYPE;
    float4 Color : COLOR0;
};

// .. VERTEX SHADERS

VS_COLORED_OUTPUT vs_2d_main(
    VS_COLORED_2D_INPUT input,
    uniform const bool it_uses_projection_matrix
    )
{
    VS_COLORED_OUTPUT
        output;
       
    if ( it_uses_projection_matrix )
    {
        output.Position = mul( float4( input.Position, 0, 1 ), ProjectionMatrix );
    }
    else
    {
        output.Position = float4( input.Position, 0, 1 );
    }
    
    output.Color = input.Color;
    
    return output;
}

// ~~

VS_COLORED_OUTPUT vs_3d_main(
    VS_COLORED_3D_INPUT input
    )
{
    VS_COLORED_OUTPUT
        output;
    float4x4
        wvp;
       
    wvp = mul( CameraMatrix, ProjectionMatrix );

    output.Position = mul( float4( input.Position, 1 ), wvp );
    output.Color = input.Color;
    
    return output;
}

// .. PIXEL SHADERS

float4 ps_main(
    #ifdef GRAPHIC_PLATFORM_Dx11
        float4 pos : POSITION_INPUT_TYPE,
    #endif
    float4 color : COLOR0
    ) : PS_OUTPUT_TYPE
{
    return color;
}

// .. TECHNIQUES

TECHNIQUE Lines2D
{
    pass P0
    {
        VertexShader = compile VS_COMPILE_VERSION vs_2d_main( true );
        PixelShader = compile PS_COMPILE_VERSION ps_main();
    }
}

// ~~

TECHNIQUE Lines2DNoProjection
{
    pass P0
    {
        VertexShader = compile VS_COMPILE_VERSION vs_2d_main( false );
        PixelShader = compile PS_COMPILE_VERSION ps_main();
    }
}

// ~~

TECHNIQUE Lines3D
{
    pass P0
    {
        VertexShader = compile VS_COMPILE_VERSION vs_3d_main();
        PixelShader = compile PS_COMPILE_VERSION ps_main();
    }
}
