float4x4
    ProjectionMatrix; 
float4x4
    ModelViewMatrix;
float4x4
    TextureCoordinateTransformMatrix;
    
// ~~

struct VS_INPUT
{
    float3 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

Texture DiffuseTexture;

sampler2D DiffuseTextureSampler
{ 
    Texture = <DiffuseTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
};

// ~~

VS_OUTPUT vs_main(
    VS_INPUT input,
    const uniform bool it_uses_coordinates_transform
    )
{
    VS_OUTPUT
        output = (VS_OUTPUT) 0;
    float4x4
        wvp;
    
    wvp = mul( ModelViewMatrix, ProjectionMatrix );
    output.Position = mul( float4( input.Position, 1 ), wvp );
    
    if ( it_uses_coordinates_transform )
    {        
        output.TextureCoordinate = ( mul( float4( input.TextureCoordinate.xy, 0.0f, 1.0f ), TextureCoordinateTransformMatrix ) ).xy;        
    }
    else
    {
        output.TextureCoordinate = input.TextureCoordinate;
    }
    
    return output;
}

// ~~

float4 ps_textured_main(
    float2 texture_coordinate : TEXCOORD0 
    ) : COLOR0
{
    return tex2D( DiffuseTextureSampler, texture_coordinate ); 
}

// ~~

technique TexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main( false );
        PixelShader = compile ps_3_0 ps_textured_main();
    }
}

// ~~

technique TransformedTexturedInput
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main( true );
        PixelShader = compile ps_3_0 ps_textured_main();
    }
}

