// .. MACROS

#ifdef GRAPHIC_PLATFORM_Dx11
    #define TECHNIQUE technique11
    #define PS_OUTPUT_TYPE SV_Target0
    #define POSITION_INPUT_TYPE SV_POSITION
    #define VS_COMPILE_VERSION vs_4_0
    #define PS_COMPILE_VERSION ps_4_0
    #define GS_COMPILE_VERSION gs_4_0
#else
    #define TECHNIQUE technique
    #define PS_OUTPUT_TYPE COLOR0
    #define POSITION_INPUT_TYPE POSITION
    #define VS_COMPILE_VERSION vs_3_0
    #define PS_COMPILE_VERSION ps_3_0
    #define GS_COMPILE_VERSION UNSUPPORTED
#endif

// .. CONSTANTS

float4x4
    ProjectionMatrix; 
float4x4
    ModelViewMatrix;
float4 
    TextureModulationColor = { 1.0, 1.0, 1.0, 1.0 };
float
    Alpha = 1.0;

// ~~

#ifdef GRAPHIC_PLATFORM_Dx11
    Texture2D
        DiffuseTexture;
#else
    Texture
        DiffuseTexture;
#endif
    
// .. TYPES

struct VS_TEXTURED_INPUT
{
    float3 Position : POSITION_INPUT_TYPE;
    float2 TextureCoordinate : TEXCOORD0;
};

// ~~

struct VS_TEXTURED_OUTPUT
{
    float4 Position : POSITION_INPUT_TYPE;
    float2 TextureCoordinate : TEXCOORD0;
};

// .. SAMPLERS

#ifdef GRAPHIC_PLATFORM_Dx11
    SamplerState DiffuseTextureSampler
    {
        Filter = MIN_MAG_MIP_LINEAR;
        AddressU = Wrap;
        AddressV = Wrap;
    };

    #define PS_SAMPLE( _TEXTURE_, _TEXTURE_COORDS_ ) \
        _TEXTURE_.Sample( _TEXTURE_##Sampler, _TEXTURE_COORDS_ )
#else
    sampler2D DiffuseTextureSampler
    { 
        Texture = <DiffuseTexture>;
        MinFilter = LINEAR;
        MagFilter = LINEAR;
        MipFilter = LINEAR;
    };

    #define PS_SAMPLE( _TEXTURE_, _TEXTURE_COORDS_ ) \
        tex2D( _TEXTURE_##Sampler, _TEXTURE_COORDS_ )
#endif

// .. OPERATIONS

VS_TEXTURED_OUTPUT vs_main(
    VS_TEXTURED_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
    float4x4
        wvp;

    wvp = mul( ModelViewMatrix, ProjectionMatrix );
        
    output.Position = mul( float4( input.Position, 1 ), wvp ) ;
    output.TextureCoordinate = input.TextureCoordinate;
    
    return output;
}

// ~~

float4 ps_main(
	#ifdef GRAPHIC_PLATFORM_Dx11
        float4 pos : POSITION_INPUT_TYPE,
    #endif
	float2 texture_coordinate : TEXCOORD0
	) : PS_OUTPUT_TYPE
{
    float4
        return_color;
        
    return_color = TextureModulationColor * PS_SAMPLE( DiffuseTexture, texture_coordinate );
    return return_color * Alpha; 
}

// .. TECHNIQUES

TECHNIQUE Default
{
    pass P0
    {
        VertexShader = compile VS_COMPILE_VERSION vs_main();
        PixelShader = compile PS_COMPILE_VERSION ps_main();
    }
}
