// -- SEMANTICS BEGIN
#define GRAPHIC_VERTEX_PARAMETER_Position POSITION
#define GRAPHIC_VERTEX_PARAMETER_DiffuseMapping0 TEXCOORD0
// -- SEMANTICS END

struct VS_INPUT_STRIP
{
	float2 Position : GRAPHIC_VERTEX_PARAMETER_Position;		
};

struct VS_INPUT_FULLSCREEN
{
	float3 Position : GRAPHIC_VERTEX_PARAMETER_Position;		
};

struct VS_INPUT_LINE
{
    float2 Position : GRAPHIC_VERTEX_PARAMETER_Position;
    float4 LineCoefficient : TEXCOORD0;	
    float LineRadius : TEXCOORD1;	
};

struct VS_OUTPUT_STRIP
{
    float4 Position : POSITION;
	float2 TextureCoordinate : TEXCOORD0;
};

struct VS_OUTPUT_FULLSCREEN
{
	float4 Position : GRAPHIC_VERTEX_PARAMETER_Position;		
};

struct VS_OUTPUT_LINE
{
    float4 Position : POSITION;
    float4 LineCoefficient : TEXCOORD0;
    float LineRadius : TEXCOORD1;	
};

struct VS_OUTPUT_STRIP_SOLID
{
    float4 Position : POSITION;
};

struct VS_OUTPUT_GLYPH
{
	float4 Position : POSITION;
	float2 TextureCoordinate : TEXCOORD0;
};

struct VS_INPUT_GLYPH
{
	float2 Position : POSITION;
	float2 TextureCoordinate : TEXCOORD0;
};

struct PS_INPUT_TEXTURE
{
	float2 TextureCoordinate : TEXCOORD0;
};

struct PS_INPUT_LINE
{
    float2 Position : VPOS;
    float4 LineCoefficient : TEXCOORD0;
    float LineRadius : TEXCOORD1;
};

float4x4 ViewportMatrix;
float4 TextureMatrixRow0;
float4 TextureMatrixRow1;
float4 Color;
float4 ColorMultiplier;
float4 ColorAddition;

texture Texture;

sampler2D TextureSampler = sampler_state
{
	Texture = <Texture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

    AddressU = CLAMP;
    AddressV = CLAMP;
    AddressW = CLAMP;
};

texture TextureLookup;

sampler TextureLookupSampler = sampler_state
{
	Texture = <TextureLookup>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;

    AddressU = CLAMP;
    AddressV = CLAMP;
    AddressW = CLAMP;
};

VS_OUTPUT_STRIP_SOLID VSStripSolid( 
    const VS_INPUT_STRIP input 
    )
{
    VS_OUTPUT_STRIP_SOLID
        output;
    
    output.Position.x = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 0 ] );
    output.Position.y = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 1 ] );
    output.Position.z = 0;
    output.Position.w = 1;
    
    return output;
}

VS_OUTPUT_FULLSCREEN VSFullScreen(
	const VS_INPUT_FULLSCREEN input
	)
{
	VS_OUTPUT_FULLSCREEN output;

	output.Position  = float4( input.Position, 1.0 );

	return output;
}

VS_OUTPUT_LINE VSLine( 
    const VS_INPUT_LINE input 
    )
{
    VS_OUTPUT_LINE
        output;
    
    output.Position.x = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 0 ] );
    output.Position.y = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 1 ] );
    output.Position.z = 0;
    output.Position.w = 1;
    
    output.LineCoefficient = input.LineCoefficient;
    output.LineRadius = input.LineRadius;
    
    return output;
} 

VS_OUTPUT_STRIP VSStripTexture( 
    const VS_INPUT_STRIP input 
    )
{
    VS_OUTPUT_STRIP
        output;
    
    output.Position.x = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 0 ] );
    output.Position.y = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 1 ] );
    output.Position.z = 0;
    output.Position.w = 1;
    output.TextureCoordinate.x = dot( float4( input.Position, 0, 1 ), TextureMatrixRow0 );
    output.TextureCoordinate.y = dot( float4( input.Position, 0, 1 ), TextureMatrixRow1 );
    
    return output;
} 

VS_OUTPUT_GLYPH VSGlyphText(
	const VS_INPUT_GLYPH input
	)
{
	VS_OUTPUT_GLYPH
		output;

    output.Position.x = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 0 ] );
    output.Position.y = dot( float4( input.Position, 0, 1 ), ViewportMatrix[ 1 ] );
    output.Position.z = 0;
    output.Position.w = 1;
    output.TextureCoordinate = input.TextureCoordinate;
	
    return output;
}

float4 PSSolid() : COLOR
{
    return Color;
    float4
        color;
        
    color.rgb = Color.rgb * Color.a;
    color.a = Color.a;
    
    return color;
}

float4 PSCxformTexture(
	const PS_INPUT_TEXTURE input
	) : COLOR
{
    float4 
        color,
        addition_color;
		
    #ifdef GRAPHIC_PLATFORM_Xbox360		
        color = tex2D( TextureSampler, input.TextureCoordinate ).argb;
    #else
        color = tex2D( TextureSampler, input.TextureCoordinate ).bgra;
    #endif
    
    color *= ColorMultiplier;
    //color.rgb *= ColorMultiplier.a;
    
    //addition_color.rgb = ColorAddition.rgb * ColorAddition.a;
    //addition_color.a = ColorAddition.a;
    //color.rgb += addition_color.rgb;
	color += ColorAddition;
    
    return color;
}

float4 PSTextTexture(
	const PS_INPUT_TEXTURE input
	) : COLOR
{
    float4
        color;
	
	// Texture in BGRA generated by us
    color = tex2D( TextureSampler, input.TextureCoordinate ).bgra;
    color *= Color;
	 
    return color;
}

float4 PSLineSolid(
    const PS_INPUT_LINE input
    ) : COLOR
{
    float 
        anti_aliasing_width = 2;
    float3 
        position;
    float 
        scaled_distance;
    float
        alpha;
 
    position = float3( input.Position.x, input.Position.y, 1.0f);

    scaled_distance = abs( dot( input.LineCoefficient.xyz, position.xyz ) );
    scaled_distance -= input.LineRadius;
    
    // discard fragments that lie outside the line
    alpha = 1.0f - saturate( scaled_distance / anti_aliasing_width );
    alpha = tex2D( TextureLookupSampler, float2( alpha, 0.5f ) ).r;

    return float4( Color.xyz , Color.w * alpha );
}

technique SolidStrip
{
	pass P0
	{
		VertexShader = compile vs_3_0 VSStripSolid();
		PixelShader = compile ps_3_0 PSSolid();
	}
}

technique SolidLine
{
	pass P0
	{
		VertexShader = compile vs_3_0 VSLine();
		PixelShader = compile ps_3_0 PSLineSolid();
	}
}

technique CxFormTextureStrip
{
	pass P0
	{
		VertexShader = compile vs_3_0 VSStripTexture();
		PixelShader = compile ps_3_0 PSCxformTexture();
	}
}

technique GlyphText
{
	pass P0
	{
		VertexShader = compile vs_3_0 VSGlyphText();
		PixelShader = compile ps_3_0 PSTextTexture();
	}
}

technique MaskDrawing
{
	pass P0
	{
		VertexShader = compile vs_3_0 VSFullScreen();
		PixelShader = NULL;
	}
}