struct VS_INPUT
{
    float2 Position : POSITION;
	float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

// ~~

struct VS_OUTPUT
{
    float4 Position : POSITION;
	float4 Color : COLOR0;
    float2 TextureCoordinate : TEXCOORD0;
};

// ~~

float4x4
    ProjectionMatrix; 
float4x4
	ModelViewMatrix;
Texture
    DiffuseTexture;

sampler2D DiffuseTextureSampler
{ 
    Texture = <DiffuseTexture>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
};
    
// ~~

VS_OUTPUT vs_main(
    VS_INPUT input
    )
{
    VS_OUTPUT
        output; 
	float4x4
		wvp;
	
	wvp = mul( ModelViewMatrix, ProjectionMatrix );
        
    output.Position = mul( float4( input.Position, 0, 1 ), wvp ) ;
	output.Color = input.Color;
    output.TextureCoordinate = input.TextureCoordinate;
    
    return output;
}

// ~~

float4 ps_main(
   float4 color : COLOR0
   ) : COLOR0
{
    return color ;
}

// ~~

float4 ps_textured_main(
   float2 texture_coordinate : TEXCOORD0,
   float4 color : COLOR0
   ) : COLOR0
{
    return color * tex2D( DiffuseTextureSampler, texture_coordinate );
}

// .. TECHNIQUES

technique Default
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main();
        PixelShader = compile ps_3_0 ps_main();
    }
}

technique Textured
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_main();
        PixelShader = compile ps_3_0 ps_textured_main();
    }
}
