// .. VARIABLES

// .. STRUCTS

// .. TEXTURES

Texture TextureY;

sampler2D TextureYSampler
{ 
    Texture = <TextureY>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = POINT;
};

// ~~

Texture TextureU;

sampler2D TextureUSampler
{ 
    Texture = <TextureU>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = POINT;
};

// ~~

Texture TextureV;

sampler2D TextureVSampler
{ 
    Texture = <TextureV>;
    MinFilter = POINT;
    MagFilter = POINT;
    MipFilter = POINT;
};

struct VS_TEXTURED_INPUT
{
    float2 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

struct VS_TEXTURED_OUTPUT
{
    float4 Position : POSITION;
    float2 TextureCoordinate : TEXCOORD0;
};

// .. VERTEX_SHADERS

VS_TEXTURED_OUTPUT vs_textured_main(
    VS_TEXTURED_INPUT input
    )
{
    VS_TEXTURED_OUTPUT
        output;
        
    output.Position = float4( input.Position, 0, 1 );
    output.TextureCoordinate = input.TextureCoordinate;
    
    return output;
}

// .. PIXEL_SHADERS

float4 ps_yuv_to_rgb_main(
   float2 texture_coordinate : TEXCOORD0 
   ) : COLOR0
{
    float
        R, G, B, Y, U, V, Ydiv;
        
    Ydiv = 16 / 255;
    
    Y = tex2D( TextureYSampler, texture_coordinate );
    U = tex2D( TextureUSampler, texture_coordinate );
    V = tex2D( TextureVSampler, texture_coordinate );
    
    R = clamp( 1.164 * ( Y - Ydiv ) + 1.596 * ( V - 0.5 ), 0.0, 1.0 );
    G = clamp( 1.164 * ( Y - Ydiv ) - 0.813 * ( V - 0.5 ) - 0.391 * ( U - 0.5 ), 0.0, 1.0 );
    B = clamp( 1.164 * ( Y - Ydiv ) + 2.018 * ( U - 0.5 ), 0.0, 1.0 );
    
    return float4( R, G, B, 1.0 );
}

// .. TECHNIQUES

technique YuvToRgb
{
    pass P0
    {
        VertexShader = compile vs_3_0 vs_textured_main();
        PixelShader = compile ps_3_0 ps_yuv_to_rgb_main();
    }
}